#include "com/amazonaws/kinesis/video/webrtcclient/Include.h"
#include "json/json.h"

#include "ws_client.h"

#include <iostream>
#include <thread>

static PRtcPeerConnection peer_;
static PRtcDataChannel dc_;
static WSClient *ws = nullptr;

static void onWsData(const std::string& data, void* usrParam) {
  Json::Reader reader;
  Json::Value root;
  if (reader.parse(data, root)) {
    if(root["type"] == "answer") {
      RtcSessionDescriptionInit answer;
      Json::FastWriter writer;
      writer.omitEndingLineFeed();
      std::string _sdp_json_answer = writer.write(root);
      int ret = deserializeSessionDescriptionInit((PCHAR)_sdp_json_answer.data(), _sdp_json_answer.size(), &answer);
      std::cout << "deserializeSessionDescriptionInit:" << ret << std::endl;
      std::cout << "answer:\n" << answer.sdp << std::endl;
      if(ret == 0) {
        std::cout << "setRemoteDescription:" << setRemoteDescription(peer_, &answer) << std::endl;
      }
    } else if (root["type"] == "candidate") {
      if(!root["candidate"].asString().empty()) {
        //std::cout << "addIceCandidate:" << addIceCandidate(peer_, "") << std::endl;
      //} else{
        RtcIceCandidateInit ice;
        Json::FastWriter writer;
        writer.omitEndingLineFeed();
        std::string _ice_json = writer.write(root);
        int ret = deserializeRtcIceCandidateInit((PCHAR)_ice_json.data(), _ice_json.size(), &ice);
        std::cout << "deserializeRtcIceCandidateInit:" << ret << std::endl;
        if(ret == 0) {
          std::cout << "remote-ice:" << ice.candidate << std::endl;
          std::cout << "addIceCandidate:" << addIceCandidate(peer_, ice.candidate) << std::endl;
        }
      }
    } else if(root["type"] == "hello") {

      RtcSessionDescriptionInit offer;
      std::cout << "createOffer:" << createOffer(peer_, &offer) << std::endl;
      std::cout << "offer:\n" << offer.sdp << std::endl;

      std::cout << "setLocalDescription:" << setLocalDescription(peer_, &offer) << std::endl;

      char sdp_json[10240] = {0};
      unsigned int sdp_json_buf_len = sizeof sdp_json;
      std::cout << "serializeSessionDescriptionInit:" << serializeSessionDescriptionInit(&offer, sdp_json, &sdp_json_buf_len) << std::endl;
      std::cout << "sdp_json_offer:\n" << sdp_json << std::endl;

      ws->Send(sdp_json);
    }
  } else {
    std::cout << "Json parse failed" << std::endl;
  }
}

void func_RtcOnIceCandidate(UINT64 custmerData, PCHAR iceStr) {
  if(iceStr != nullptr && strlen(iceStr) > 0) {
    std::cout << "local-ice:" << iceStr << std::endl;
    Json::Value ice;
    Json::Reader reader;
    if(reader.parse(iceStr, ice)) {
      ice["type"] = "candidate";
      Json::FastWriter writer;
      writer.omitEndingLineFeed();
      std::string _ice_json = writer.write(ice);
      ws->Send(_ice_json);
    }
  }
}

void func_RtcOnMessage(UINT64 customData, PRtcDataChannel dc, BOOL isBinary, PBYTE msgData, UINT32 msgLen) {
  printf("func_RtcOnMessage dc_label=%s isBinary=%d msgLen=%d\n", dc->name, isBinary, msgLen);
  if(!isBinary) {
    printf("\t%.*s\n", msgLen, msgData);
  }
}

void func_RtcOnConnectionStateChange(UINT64 customData, RTC_PEER_CONNECTION_STATE state) {
  if(state == RTC_PEER_CONNECTION_STATE::RTC_PEER_CONNECTION_STATE_CONNECTED) {
    RtcDataChannelInit dcInit;
    memset(&dcInit, 0, sizeof dcInit);
    dcInit.ordered = TRUE;
    std::cout << "createDataChannel:" << createDataChannel(peer_, (PCHAR)"op", &dcInit, &dc_) << std::endl;
    std::cout << "dataChannelOnMessage:" << dataChannelOnMessage(dc_, 0, func_RtcOnMessage) << std::endl;
  }
}

int main() {
  //printf("STATUS_ICE_CANDIDATE_INIT_MALFORMED %lu\n", STATUS_ICE_CANDIDATE_INIT_MALFORMED);

  char wsUrl[64];  
  sprintf(wsUrl, "ws://192.168.0.44:8080/webrtc/channel/01239F-C8ADE4-630DEE");
  ws = new WSClient(wsUrl);

  ws->SetDataCb(onWsData, nullptr);
  if(!ws->Start()) return -1;

  initKvsWebRtc();

  RtcConfiguration conf_;
  memset(&conf_, 0, sizeof conf_);
  conf_.iceTransportPolicy = ICE_TRANSPORT_POLICY::ICE_TRANSPORT_POLICY_ALL;
  //conf_.kvsRtcConfiguration.generatedCertificateBits = 2048;
  conf_.kvsRtcConfiguration.generateRSACertificate = 1;
  strcpy(conf_.iceServers[0].urls, "stun:77.72.169.213:3478");
  std::cout << "createPeerConnection:" << createPeerConnection(&conf_, &peer_) << std::endl;

  std::cout << "peerConnectionOnIceCandidate:" << peerConnectionOnIceCandidate(peer_, 0, func_RtcOnIceCandidate) << std::endl;
  std::cout << "peerConnectionOnConnectionStateChange:" << peerConnectionOnConnectionStateChange(peer_, 0, func_RtcOnConnectionStateChange) << std::endl;

  PRtcMediaStreamTrack video_track_ = new RtcMediaStreamTrack;
  memset(video_track_, 0, sizeof *video_track_);
  video_track_->codec = RTC_CODEC::RTC_CODEC_H264_PROFILE_42E01F_LEVEL_ASYMMETRY_ALLOWED_PACKETIZATION_MODE;
  video_track_->kind = MEDIA_STREAM_TRACK_KIND::MEDIA_STREAM_TRACK_KIND_VIDEO;
  strcpy(video_track_->streamId, "0");
  strcpy(video_track_->trackId, "0");
  PRtcRtpTransceiver videoReceiver;
  RtcRtpTransceiverInit rtcRtpTransceiverInit_v_;
  rtcRtpTransceiverInit_v_.direction = RTC_RTP_TRANSCEIVER_DIRECTION::RTC_RTP_TRANSCEIVER_DIRECTION_SENDONLY;
  std::cout << "addTransceiver:" << addTransceiver(peer_, video_track_, &rtcRtpTransceiverInit_v_, &videoReceiver) << std::endl;

  PRtcMediaStreamTrack audio_track_ = new RtcMediaStreamTrack;
  memset(audio_track_, 0, sizeof *audio_track_);
  audio_track_->codec = RTC_CODEC::RTC_CODEC_ALAW;
  audio_track_->kind = MEDIA_STREAM_TRACK_KIND::MEDIA_STREAM_TRACK_KIND_AUDIO;
  strcpy(audio_track_->streamId, "1");
  strcpy(audio_track_->trackId, "1");
  PRtcRtpTransceiver audioReceiver;
  RtcRtpTransceiverInit rtcRtpTransceiverInit_a_;
  rtcRtpTransceiverInit_a_.direction = RTC_RTP_TRANSCEIVER_DIRECTION::RTC_RTP_TRANSCEIVER_DIRECTION_SENDRECV;
  std::cout << "addTransceiver:" << addTransceiver(peer_, audio_track_, &rtcRtpTransceiverInit_a_, &audioReceiver) << std::endl;

  while(true) {
    std::this_thread::sleep_for(std::chrono::seconds(1));
  }
  deinitKvsWebRtc();

  return -1;
}