#include "ws_client.h"

#include "mongoose.h"

#include <iostream>
#include <iomanip>

#define DEBUG_ 0
void WSClient::ev_handler_mg_(mg_connection *nc, int ev, void *ev_data)
{
    WSClient *client_ = (WSClient *)nc->user_data;
    if(client_ == nullptr) return;
    switch (ev)
    {
    case MG_EV_WEBSOCKET_HANDSHAKE_DONE:
    {
        struct http_message *hm = (struct http_message *)ev_data;
        if (nullptr == hm || hm->resp_code == 101)
        {
            client_->connectedFlag_ = true;
        }
        break;
    }
    case MG_EV_POLL:
    {
        if(client_->heartBeatIntervalSec_ > 0 && mg_time() - client_->lastHeartBeatTime_ > client_->heartBeatIntervalSec_) {
            mg_send_websocket_frame(nc, WEBSOCKET_OP_TEXT, client_->heartBeatBody_.data(), client_->heartBeatBody_.size());
            client_->lastHeartBeatTime_ = mg_time();
        }
        break;
    }
    case MG_EV_WEBSOCKET_FRAME: 
    {
        websocket_message *wm = (websocket_message *) ev_data;
        // 当前使用的mongoose版本中，OPCODE是不对的，会变成CONTINUE。查阅github最新版本发现修复过，但无法合并代码，怕出问题
        /*
        int _opcode = wm->flags & 0x0F;
        if(_opcode == WEBSOCKET_OP_TEXT) {
            if(wm->size < 255)
                std::cout << "recv:" << std::string((const char *)wm->data, (int)wm->size) << std::endl;
            else
                std::cout << "recv:" << std::string((const char *)wm->data, 255) << "..." << std::endl;
        } else {
            int _cout_size = wm->size > 255 ? 255 : wm->size;
            std::cout << "recv:";
            for (int i = 0; i < _cout_size; ++i) {
                std::cout << std::hex << (int)wm->data[i];
                if(i != _cout_size - 1) std::cout << " ";
            }
            if(wm->size > 255) std::cout << "...";
            std::cout << std::endl;
        }*/
        // 由于websocket的文本是utf8，因此可能出现中文显示不正确的问题（会截断，以及isprint判断不正确）
        if(DEBUG_) {
            int _cout_size = wm->size > 255 ? 255 : wm->size;
            std::cout << "recv:";
            for (int i = 0; i < _cout_size; ++i) {
                if(isprint((int)wm->data[i]))
                    std::cout << wm->data[i];
                else {
                    std::cout << std::hex << (int)wm->data[i];
                    if(i != _cout_size - 1) std::cout << " ";
                }
            }
            if(wm->size > 255) std::cout << "...";
            std::cout << std::endl;
        }
        {
            std::unique_lock<std::mutex> cvlk(client_->mut_);
            client_->recv_ = std::string((const char *)wm->data, (int)wm->size);
            client_->cv_.notify_all();
        }
        std::lock_guard<std::mutex> lk(client_->dispatch_mtx_);
        client_->wsDatas_.push_back(std::string((const char *)wm->data, wm->size));
        /*if(client_->on_ws_data_cb_ != nullptr) {
            std::thread([](WSClient *this_, std::string ws_data_){
                this_->on_ws_data_cb_(ws_data_, this_->data_cb_usrParam_);
            }, client_, std::string((const char *)wm->data, wm->size)).detach();
        }*/
        break;
    }
    case MG_EV_CLOSE:
    {
        client_->connectedFlag_ = false;
        if(client_->on_ws_disconnect_cb_ != nullptr) {
            client_->on_ws_disconnect_cb_(client_->disconnect_cb_usrParam_);
        }
        break;
    }
    }
}

WSClient::WSClient(std::string url, bool autoReconnect, int heartBeatIntervalSec, const std::string &heartBeatBody)
    : url_(url)
    , heartBeatIntervalSec_(heartBeatIntervalSec)
    , heartBeatBody_(heartBeatBody)
    , autoReconnect_(autoReconnect)
    , exitFlag_(true)
    , connectedFlag_(false)
    , on_ws_data_cb_(nullptr)
    , data_cb_usrParam_(nullptr)
    , on_ws_disconnect_cb_(nullptr)
    , disconnect_cb_usrParam_(nullptr)
    , lastHeartBeatTime_(0)
{
}

bool WSClient::Start()
{
    std::lock_guard<std::recursive_mutex> lk(mutex_);
    if(connectedFlag_) return true;
    if(!exitFlag_) return false;
    mg_mgr *mgr_ = (mg_mgr *)malloc(sizeof(mg_mgr));
    mg_mgr_init(mgr_, nullptr);
    mg_connection *$nc_ = mg_connect_ws(mgr_, WSClient::ev_handler_mg_, url_.c_str(), nullptr, nullptr);
    int c = 0;
    if (nullptr == $nc_)
        goto lbl_fail;
    $nc_->user_data = this;
    while(!this->connectedFlag_ && ++c < 10) {
        std::this_thread::sleep_for(std::chrono::milliseconds(300));
        mg_mgr_poll(mgr_, 0);
    }
    if(!this->connectedFlag_)
        goto lbl_fail;
    this->nc_ = $nc_;
    exitFlag_ = false;
    std::thread(WSClient::mgRun, this, mgr_).detach();
    std::thread(WSClient::wsDataDispatch, this).detach();
    if(false) {
lbl_fail:
        mg_mgr_free(mgr_);
        free(mgr_);
        return false;
    }
    return true;
}

void WSClient::mgRun(WSClient * this_, mg_mgr *mgr_) {
    while(true) {
        std::this_thread::sleep_for(std::chrono::milliseconds(100)); // 必须休眠，不然锁竞争不会让出当前线程
        std::lock_guard<std::recursive_mutex> lk(this_->mutex_);
        if(this_->exitFlag_) break;
        mg_mgr_poll(mgr_, 0);
        if(!this_->connectedFlag_)
            if (this_->autoReconnect_) {
                if(mgr_->active_connections != nullptr) { // 如果已经断开但连接并未被移除，则让mongoose下次将其移除
                    mgr_->active_connections->flags |= MG_F_CLOSE_IMMEDIATELY;
                }
                mg_connection *$nc_ = mg_connect_ws(mgr_, WSClient::ev_handler_mg_, this_->url_.c_str(), nullptr, nullptr);
                if (nullptr == $nc_)
                {
                    std::this_thread::sleep_for(std::chrono::seconds(3));
                } else {
                    $nc_->user_data = this_;
                    int c = 0;
                    while(!this_->connectedFlag_ && ++c < 10) {
                        std::this_thread::sleep_for(std::chrono::milliseconds(300));
                        mg_mgr_poll(mgr_, 0);
                    }
                }
                this_->nc_ = $nc_;
            } else {
                this_->exitFlag_ = true;
            }
    }
    mg_mgr_free(mgr_);
    free(mgr_);
}

void WSClient::wsDataDispatch(WSClient *this_) {
     while(true) {
        std::this_thread::sleep_for(std::chrono::milliseconds(100));
        std::lock_guard<std::mutex> lk(this_->dispatch_mtx_);
        if(this_->exitFlag_) break;
        if(this_->on_ws_data_cb_ != nullptr && !this_->wsDatas_.empty()) {
            for(std::vector<std::string>::iterator iter = this_->wsDatas_.begin(); iter != this_->wsDatas_.end(); iter++) {
                this_->on_ws_data_cb_(*iter, this_->data_cb_usrParam_);
            }
            this_->wsDatas_.clear();
        }
     }
}

void WSClient::Stop() {
    std::lock_guard<std::recursive_mutex> lk(mutex_);
    if(exitFlag_) return;
    exitFlag_ = true;
    connectedFlag_ = false;
    if(nc_ != nullptr)
        nc_->user_data = nullptr;
}

WSClient::~WSClient() {
    Stop();
}

void WSClient::SetDisconnectCb(on_ws_disconnect_cb disconnect_cb, void* usrParam) {
    std::lock_guard<std::recursive_mutex> lk(mutex_);
    on_ws_disconnect_cb_ = disconnect_cb;
    disconnect_cb_usrParam_ = usrParam;
}

void WSClient::SetDataCb(on_ws_data_cb data_cb, void* usrParam) {
    std::lock_guard<std::recursive_mutex> lk(mutex_);
    on_ws_data_cb_ = data_cb;
    data_cb_usrParam_ = usrParam;
}

void WSClient::Send(const std::string &data2send) {    
    if (data2send.empty() || data2send.size() < 1 || data2send.size() >= 1024*1024*10) { return; }

    std::lock_guard<std::recursive_mutex> lk(mutex_);
    if(exitFlag_ || !connectedFlag_) return;
    if(DEBUG_) std::cout << "send:" << data2send << std::endl;
    mg_send_websocket_frame(nc_, WEBSOCKET_OP_TEXT, data2send.data(), data2send.size() );
}

bool WSClient::IsConnected() {
    std::lock_guard<std::recursive_mutex> lk(mutex_);
    return connectedFlag_;
}

void WSClient::Request(const std::string &req, std::string & resp, unsigned int timeoutms) {
    if (req.empty() || req.size() < 1 || req.size() >= 1024*1024*10) { return; }

    {
        std::lock_guard<std::recursive_mutex> lk(mutex_);
        if(exitFlag_ || !connectedFlag_) return;
        if(DEBUG_) std::cout << "send:" << req << std::endl;
        mg_send_websocket_frame(nc_, WEBSOCKET_OP_TEXT, req.data(), req.size() );
    }

    recv_.clear();
    std::unique_lock<std::mutex> cvlk(mut_);

    if (!cv_.wait_for(cvlk, std::chrono::milliseconds(timeoutms), 
        [&]() { return !recv_.empty(); })) {
        return;
    }

    resp = recv_;
    recv_.clear();
}

void WSClient::Request(const std::string &req, std::string & resp, judge_request_success judge_fun, unsigned int timeoutms) {
    if (req.empty() || req.size() < 1 || req.size() >= 1024*1024*10) { return; }

    {
        std::lock_guard<std::recursive_mutex> lk(mutex_);
        if(exitFlag_ || !connectedFlag_) return;
        std::cout << "send:" << req << std::endl;
        mg_send_websocket_frame(nc_, WEBSOCKET_OP_TEXT, req.data(), req.size() );
    }

    recv_.clear();
    std::unique_lock<std::mutex> cvlk(mut_);

    if (!cv_.wait_for(cvlk, std::chrono::milliseconds(timeoutms), 
        [&]() { return !recv_.empty() && judge_fun(recv_); })) {
        return;
    }

    resp = recv_;
    recv_.clear();
}

void WSClient::Request(const char *req, size_t data_len, std::string & resp, judge_request_success judge_fun, unsigned int timeoutms) {
    if (req == nullptr || data_len >= 1024*1024*10) { return; }

    {
        std::lock_guard<std::recursive_mutex> lk(mutex_);
        if(exitFlag_ || !connectedFlag_) return;
        std::cout << "send:" << std::string(req, data_len) << std::endl;
        mg_send_websocket_frame(nc_, WEBSOCKET_OP_BINARY, req, data_len );
    }

    recv_.clear();
    std::unique_lock<std::mutex> cvlk(mut_);

    if (!cv_.wait_for(cvlk, std::chrono::milliseconds(timeoutms), 
        [&]() { return !recv_.empty() && judge_fun(recv_); })) {
        return;
    }

    resp = recv_;
    recv_.clear();
}