#ifndef WS_CLIENT_H_
#define WS_CLIENT_H_

#include <string>
#include <mutex>
#include <condition_variable>
#include <thread>
#include <functional>
#include <atomic>
#include <vector>

typedef void (*on_ws_disconnect_cb) (void*);

typedef void (*on_ws_data_cb) (const std::string&, void* usrParam);

using judge_request_success = std::function<bool(const std::string &)>;

struct mg_connection;
struct mg_mgr;
class WSClient {
public:
WSClient(std::string url, bool autoReconnect = true, int heartBeatIntervalSec = 0, const std::string &heartBeatBody = "");
bool Start();
void Stop();
~WSClient();
void SetDisconnectCb(on_ws_disconnect_cb, void*);
void SetDataCb(on_ws_data_cb, void*);
void Send(const std::string &);
bool IsConnected();

void Request(const std::string &, std::string &, unsigned int timeoutms = 3000);
void Request(const std::string &, std::string &, judge_request_success, unsigned int timeoutms = 3000);
void Request(const char *, size_t data_len, std::string &, judge_request_success, unsigned int timeoutms = 3000);

private:
on_ws_disconnect_cb on_ws_disconnect_cb_;
void* disconnect_cb_usrParam_;
on_ws_data_cb on_ws_data_cb_;
void* data_cb_usrParam_;
int heartBeatIntervalSec_;
std::string heartBeatBody_;
std::string url_;
bool autoReconnect_;
std::recursive_mutex mutex_;
bool exitFlag_;
bool connectedFlag_;
static void ev_handler_mg_(mg_connection *nc, int ev, void *ev_data);
mg_connection *nc_;
double lastHeartBeatTime_;

static void mgRun(WSClient *, mg_mgr *);

std::mutex dispatch_mtx_;
std::vector<std::string> wsDatas_;
static void wsDataDispatch(WSClient *);

std::string recv_;
std::mutex mut_;
std::condition_variable cv_;
};

#endif